"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Client = void 0;
// Modules
const events_1 = require("events");
const Websocket_1 = require("./Websocket");
const index_1 = __importDefault(require("./Rest/index"));
// Collections
const House_1 = require("./Collections/House");
const Message_1 = require("./Collections/Message");
const Member_1 = require("./Collections/Member");
const Room_1 = require("./Collections/Room");
const User_1 = require("./Collections/User");
class Client extends events_1.EventEmitter {
    constructor(options = { type: 'bot' }) {
        // Call parent class constructor
        super();
        this.options = options;
        // WS
        this.ws = new Websocket_1.WS();
        // Rest
        this.rest = new index_1.default(this);
        exports.rest = this.rest;
        exports.client = this;
        this.users = new User_1.User(this);
        this.rooms = new Room_1.Room(this);
        this.houses = new House_1.House(this);
        this.members = new Member_1.Member(this);
        this.messages = new Message_1.Message(this);
    }
    Connect(token) {
        return __awaiter(this, void 0, void 0, function* () {
            // Set the token in the instance
            if (this.options.type === 'bot')
                this.token = `Bot ${token}`;
            else
                this.token = token;
            this.rest.init(this);
            // Connect to the socket
            yield this.ws.init();
            // Auth with the socket
            yield this.ws.sendOp(2, { token: this.token });
            this.ws.on('data', (body) => {
                var _a, _b, _c, _d;
                const { e, d } = body;
                // Emits every event to RAW
                let raw = {
                    event: e,
                    description: d
                };
                this.emit('RAW', raw);
                // Detect if it's the init event to set the user in the instance
                switch (e) {
                    case 'INIT_STATE': {
                        this.user = d.user;
                        this.users.Set(d.user.id, d.user);
                        d.private_rooms.forEach((room) => {
                            this.rooms.Collect(room.id, {
                                id: room.id,
                                name: room.name,
                                house: room.house_id
                            });
                        });
                        this.emit('init', d);
                        return this.emit(e, d);
                    }
                    case 'ROOM_CREATE': {
                        let createRoomHouse = this.houses.Resolve(d.house_id);
                        (_a = createRoomHouse.rooms) === null || _a === void 0 ? void 0 : _a.Collect(d.id, {
                            id: d.id,
                            name: d.name,
                            house: createRoomHouse
                        });
                        this.rooms.Collect(d.id, {
                            id: d.id,
                            name: d.name,
                            house: createRoomHouse
                        });
                        this.houses.Collect((createRoomHouse === null || createRoomHouse === void 0 ? void 0 : createRoomHouse.id) || '', createRoomHouse);
                        this.emit('room_create', d);
                        return this.emit(e, d);
                    }
                    case 'ROOM_UPDATE': {
                        this.emit('room_update', d);
                        return this.emit(e, d);
                    }
                    case 'ROOM_DELETE': {
                        let rooms = (_b = this.houses.Get(d.house_id).rooms) === null || _b === void 0 ? void 0 : _b.Get(d.id);
                        this.houses.Get(d.house_id).rooms = rooms;
                        this.rooms.delete(d.id);
                        this.emit('room_delete', d);
                        return this.emit(e, d);
                    }
                    case 'MESSAGE_CREATE': {
                        let house = this.houses.Resolve(d.house_id);
                        const collect_message = this.messages.Collect(d.id, {
                            id: d.id,
                            content: d.content,
                            timestamp: new Date(d.timestamp),
                            room: this.rooms.Resolve(d.room_id),
                            house: this.houses.Resolve(d.house_id),
                            author: this.users.Resolve(d.author_id),
                            member: (_c = house === null || house === void 0 ? void 0 : house.members) === null || _c === void 0 ? void 0 : _c.Resolve(d.author_id)
                        });
                        this.emit('message', collect_message);
                        return this.emit(e, collect_message);
                    }
                    case 'MESSAGE_UPDATE': {
                        this.emit('message_update', d);
                        return this.emit(e, d);
                    }
                    case 'MESSAGE_DELETE': {
                        this.messages.Delete(d.id);
                        this.emit('message_delete', d);
                        return this.emit(e, d);
                    }
                    case 'TYPING_START': {
                        this.emit('typing_start', d);
                        return this.emit(e, d);
                    }
                    case 'HOUSE_JOIN': {
                        let joinHouse = d;
                        let house = this.houses.Resolve(joinHouse.id);
                        if (!house)
                            house = this.houses.Collect(joinHouse.id, {
                                id: joinHouse.id,
                                name: joinHouse.name,
                                members: new Member_1.Member(exports.client),
                                rooms: new Room_1.Room(exports.client)
                            });
                        d.rooms.forEach((room) => __awaiter(this, void 0, void 0, function* () {
                            var _e;
                            let finalRoom = {
                                id: room.id,
                                name: room.name,
                                house,
                                position: room.position,
                                type: room.type,
                                description: room.description,
                                permission_overwrites: room.permission_overwrites,
                                recipients: room.recipients,
                                typing: room.typing,
                                last_message_id: room.last_message_id,
                                emoji: room.emoji
                            };
                            this.rooms.Collect(room.id, finalRoom);
                            (_e = house.rooms) === null || _e === void 0 ? void 0 : _e.Collect(room.id, finalRoom);
                        }));
                        d.members.forEach((member) => __awaiter(this, void 0, void 0, function* () {
                            var _f;
                            (_f = house.members) === null || _f === void 0 ? void 0 : _f.Collect(member.id, {
                                id: member.id,
                                roles: member.roles,
                                last_message_id: member.last_message_id,
                                house,
                                user: this.users.Resolve(member.id)
                            });
                            this.users.Collect(member.id, member.user);
                            this.members.Collect(member.id, member.user);
                        }));
                        // Cache houses
                        this.houses.Collect(house.id || '', {
                            id: house.id,
                            name: house.name,
                            owner: this.users.Resolve(house.owner_id || ''),
                            icon: house.icon,
                            members: house.members,
                            rooms: house.rooms,
                            banner: house.banner,
                            synced: house.synced
                        });
                        this.emit('house_join', house);
                        return this.emit(e, house);
                    }
                    case 'HOUSE_MEMBER_JOIN': {
                        let houseJoin = this.houses.Get(d.house_id);
                        let member = d;
                        (_d = houseJoin.members) === null || _d === void 0 ? void 0 : _d.Collect(member.id, {
                            id: member.id,
                            house: houseJoin
                        });
                        this.houses.Collect(houseJoin.id || '', houseJoin);
                        this.users.Set(d.user_id, d.user);
                        this.emit('house_member_join', d);
                        return this.emit(e, member);
                    }
                    case 'HOUSE_MEMBER_LEAVE': {
                        // Soon
                        this.emit('house_member_leave', d);
                        return this.emit(e, d);
                    }
                    default: {
                        break;
                    }
                }
                if (body && d)
                    this.emit(e, d);
            });
            return true;
        });
    }
    destroy() {
        return __awaiter(this, void 0, void 0, function* () {
            this.ws.destroy();
        });
    }
}
exports.Client = Client;
//# sourceMappingURL=Client.js.map